@@ -32,138 +32,67 @@
 # ===================================================================
 
 """
-Digital Signature Standard (DSS), as specified in `FIPS PUB 186`__.
+Digital Signature Standard (DSS), as specified in `FIPS PUB 186-3`__.
 
 A sender signs a message in the following way:
 
         >>> from Crypto.Hash import SHA256
-        >>> from Crypto.PublicKey import DSA
+        >>> from Crypto.PublicKey import ECC
         >>> from Crypto.Signature import DSS
         >>>
         >>> message = b'I give my permission to order #4355'
-        >>> key = DSA.importKey(open('privkey.der').read())
+        >>> key = ECC.import_key(open('privkey.der').read())
         >>> h = SHA256.new(message)
         >>> signer = DSS.new(key, 'fips-186-3')
         >>> signature = signer.sign(h)
 
 The receiver can verify authenticity of the message:
 
-        >>> key = DSA.importKey(open('pubkey.der').read())
+        >>> key = ECC.import_key(open('pubkey.der').read())
         >>> h = SHA256.new(received_message)
         >>> verifier = DSS.new(key, 'fips-186-3')
-        >>> if verifier.verify(h, signature):
-        >>>    print "The signature is authentic."
-        >>> else:
-        >>>    print "The signature is not authentic."
+        >>> try:
+        >>>     verifier.verify(h, signature):
+        >>>     print "The message is authentic."
+        >>> except ValueError:
+        >>>     print "The message is not authentic."
 
 .. __: http://csrc.nist.gov/publications/fips/fips186-3/fips_186-3.pdf
 
 """
 
-__all__ = ['new', 'DSS_SigScheme']
+__all__ = ['new', 'DssSigScheme']
+
+from Crypto.Util.py3compat import bchr, b
 
-from Crypto.Util.py3compat import *
 
-from Crypto import Random
-from Crypto.Random.random import StrongRandom
 from Crypto.Util.asn1 import DerSequence
-from Crypto.Util.number import size as bit_size, long_to_bytes, bytes_to_long
+from Crypto.Util.number import long_to_bytes
+from Crypto.Math.Numbers import Integer
 
 from Crypto.Hash import HMAC
+from Crypto.PublicKey.ECC import _curve, EccKey
 
 
-class DSS_SigScheme(object):
-    """This signature scheme can perform DSS signature or verification."""
+class DssSigScheme(object):
+    """This signature scheme can perform DSS signature or verification.
 
-    #: List of L (bit length of p) and N (bit length of q) combinations
-    #: that are allowed by FIPS 186-3.
-    fips_186_3_ln = ((1024, 160), (2048, 224), (2048, 256), (3072, 256))
+    :undocumented: __init__
+    """
 
-    def __init__(self, key, mode, encoding='binary', randfunc=None):
-        """Initialize this Digital Signature Standard object.
+    def __init__(self, key, encoding, order):
+        """Create a new Digital Signature Standard (DSS) object.
 
-        :Parameters:
-          key : a DSA key object
-            If the key has the private half, both signature and
-            verification are possible.
-            If it only has the public half, verification is possible
-            but not signature generation.
-
-            Let *L* and *N* be the bit lengths of the modules *p* and *q*.
-
-            If mode is *'fips-186-3'*,
-            the combination *(L,N)* must apper in the following list,
-            in compliance to section 4.2 of `FIPS-186`__:
-
-            - (1024, 160)
-            - (2048, 224)
-            - (2048, 256)
-            - (3072, 256)
-
-            Note that the FIPS specification also defines how the key
-            must be generated. PyCrypto does not generate DSA keys
-            in a FIPS compliant way.
-
-          mode : string
-            The parameter can take these values:
-
-            - *'fips-186-3'*. The signature generation is carried out
-              according to `FIPS-186`__: the nonce *k* is taken from the RNG.
-            - *'deterministic-rfc6979'*. The signature generation
-              process does not rely on a random generator.
-              See RFC6979_.
-
-          encoding : string
-            How the signature is encoded. This value determines the output of
-            ``sign`` and the input of ``verify``.
-
-            The following values are accepted:
-
-            - *'binary'*, the signature is the raw concatenation
-              of *r* and *s*. The size in bytes of the signature is always
-              two times the size of *q*.
-
-            - *'der'*, the signature is a DER encoded SEQUENCE with two
-              INTEGERs, *r* and *s*. The size of the signature is variable.
-
-          randfunc : callable
-            The source of randomness. If `None`, the internal RNG is used.
-            It is not used under mode *'deterministic-rfc6979'*.
-
-        .. __: http://csrc.nist.gov/publications/fips/fips186-3/fips_186-3.pdf
-        .. __: http://csrc.nist.gov/publications/fips/fips186-3/fips_186-3.pdf
-        .. _RFC6979: http://tools.ietf.org/html/rfc6979
+        Do not instantiate this object directly,
+        use `Crypto.Signature.DSS.new` instead.
         """
 
-        # The goal of the 'mode' parameter is to avoid to
-        # have the current version of the standard as default.
-        #
-        # Over time, such version will be superseded by (for instance)
-        # FIPS 186-4 and it will be odd to have -3 as default.
-
-        self._deterministic = False
-        if mode == 'deterministic-rfc6979':
-            self._deterministic = True
-        elif mode not in ('fips-186-3', ):
-            raise ValueError("Unknown DSS mode '%s'" % mode)
-
-        if encoding not in ('binary', 'der'):
-            raise ValueError("Unknown encoding '%s'" % encoding)
-
-        if randfunc is None:
-            randfunc = Random.get_random_bytes
-
-        self._encoding = encoding
-        self._randfunc = randfunc
-
-        # Verify that lengths of p and q are standard compliant
-        self._l, self._n = [(bit_size(x) + 7) >> 3 for x in (key.p, key.q)]
-        if not self._deterministic:
-            if (self._l * 8, self._n * 8) not in self.fips_186_3_ln:
-                raise ValueError("L/N (%d, %d) is not compliant"
-                                 " to FIPS 186-3" %
-                                 (self._l, self._n))
         self._key = key
+        self._encoding = encoding
+        self._order = order
+
+        self._order_bits = self._order.size_in_bits()
+        self._order_bytes = (self._order_bits - 1) // 8 + 1
 
     def can_sign(self):
         """Return True if this signature object can be used
@@ -171,11 +100,121 @@ class DSS_SigScheme(object):
 
         return self._key.has_private()
 
+    def _compute_nonce(self, msg_hash):
+        raise NotImplementedError("To be provided by subclasses")
+
+    def _valid_hash(self, msg_hash):
+        raise NotImplementedError("To be provided by subclasses")
+
+    def sign(self, msg_hash):
+        """Produce the DSS signature of a message.
+
+        :Parameters:
+          msg_hash : hash object
+            The hash that was carried out over the message.
+            The object belongs to the `Crypto.Hash` package.
+
+            Under mode *'fips-186-3'*, the hash must be a FIPS
+            approved secure hash (SHA-1 or a member of the SHA-2 family),
+            of cryptographic strength appropriate for the DSA key.
+            For instance, a 3072/256 DSA key can only be used
+            in combination with SHA-512.
+
+        :Return: The signature encoded as a byte string.
+        :Raise ValueError:
+            If the hash algorithm is incompatible to the DSA key.
+        :Raise TypeError:
+            If the DSA key has no private half.
+        """
+
+        if not self._valid_hash(msg_hash):
+            raise ValueError("Hash is not sufficiently strong")
+
+        # Generate the nonce k (critical!)
+        nonce = self._compute_nonce(msg_hash)
+
+        # Perform signature using the raw API
+        z = Integer.from_bytes(msg_hash.digest()[:self._order_bytes])
+        sig_pair = self._key._sign(z, nonce)
+
+        # Encode the signature into a single byte string
+        if self._encoding == 'binary':
+            output = b("").join([long_to_bytes(x, self._order_bytes)
+                                 for x in sig_pair])
+        else:
+            # Dss-sig  ::=  SEQUENCE  {
+            #               r       OCTET STRING,
+            #               s       OCTET STRING
+            # }
+            output = DerSequence(sig_pair).encode()
+
+        return output
+
+    def verify(self, msg_hash, signature):
+        """Verify that a certain DSS signature is authentic.
+
+        This function checks if the party holding the private half of the key
+        really signed the message.
+
+        :Parameters:
+          msg_hash : hash object
+            The hash that was carried out over the message.
+            This is an object belonging to the `Crypto.Hash` module.
+
+            Under mode *'fips-186-3'*, the hash must be a FIPS
+            approved secure hash (SHA-1 or a member of the SHA-2 family),
+            of cryptographic strength appropriate for the DSA key.
+            For instance, a 3072/256 DSA key can only be used in
+            combination with SHA-512.
+
+          signature : byte string
+            The signature that needs to be validated.
+
+        :Raise ValueError:
+            If the signature is not authentic.
+        """
+
+        if not self._valid_hash(msg_hash):
+            raise ValueError("Hash does not belong to SHS")
+
+        if self._encoding == 'binary':
+            if len(signature) != (2 * self._order_bytes):
+                raise ValueError("The signature is not authentic (length)")
+            r_prime, s_prime = [Integer.from_bytes(x)
+                                for x in (signature[:self._order_bytes],
+                                          signature[self._order_bytes:])]
+        else:
+            try:
+                der_seq = DerSequence().decode(signature)
+            except (ValueError, IndexError):
+                raise ValueError("The signature is not authentic (DER)")
+            if len(der_seq) != 2 or not der_seq.hasOnlyInts():
+                raise ValueError("The signature is not authentic (DER content)")
+            r_prime, s_prime = der_seq[0], der_seq[1]
+
+        if not (0 < r_prime < self._order) or not (0 < s_prime < self._order):
+            raise ValueError("The signature is not authentic (d)")
+
+        z = Integer.from_bytes(msg_hash.digest()[:self._order_bytes])
+        result = self._key._verify(z, (r_prime, s_prime))
+        if not result:
+            raise ValueError("The signature is not authentic")
+        # Make PyCrypto code to fail
+        return False
+
+
+class DeterministicDsaSigScheme(DssSigScheme):
+    # Also applicable to ECDSA
+
+    def __init__(self, key, encoding, order, private_key):
+        super(DeterministicDsaSigScheme, self).__init__(key, encoding, order)
+        self._private_key = private_key
+
     def _bits2int(self, bstr):
         """See 2.3.2 in RFC6979"""
 
-        result = bytes_to_long(bstr)
-        q_len = bit_size(self._key.q)
+        result = Integer.from_bytes(bstr)
+        q_len = self._order.size_in_bits()
         b_len = len(bstr) * 8
         if b_len > q_len:
             result >>= (b_len - q_len)
@@ -184,18 +223,17 @@ class DSS_SigScheme(object):
     def _int2octets(self, int_mod_q):
         """See 2.3.3 in RFC6979"""
 
-        if not (0 < int_mod_q < self._key.q):
-            raise ValueError("Wrong input to int2octets()")
-        return long_to_bytes(int_mod_q, self._n)
+        assert 0 < int_mod_q < self._order
+        return long_to_bytes(int_mod_q, self._order_bytes)
 
     def _bits2octets(self, bstr):
         """See 2.3.4 in RFC6979"""
 
         z1 = self._bits2int(bstr)
-        if z1 < self._key.q:
+        if z1 < self._order:
             z2 = z1
         else:
-            z2 = z1 - self._key.q
+            z2 = z1 - self._order
         return self._int2octets(z2)
 
     def _compute_nonce(self, mhash):
@@ -213,13 +251,13 @@ class DSS_SigScheme(object):
             # Step d/f
             nonce_k = HMAC.new(nonce_k,
                                mask_v + bchr(int_oct) +
-                               self._int2octets(self._key.x) +
+                               self._int2octets(self._private_key) +
                                self._bits2octets(h1), mhash).digest()
             # Step e/g
             mask_v = HMAC.new(nonce_k, mask_v, mhash).digest()
 
         nonce = -1
-        while not (0 < nonce < self._key.q):
+        while not (0 < nonce < self._order):
             # Step h.C (second part)
             if nonce != -1:
                 nonce_k = HMAC.new(nonce_k, mask_v + bchr(0),
@@ -230,7 +268,7 @@ class DSS_SigScheme(object):
             mask_t = b("")
 
             # Step h.B
-            while len(mask_t) < self._n:
+            while len(mask_t) < self._order_bytes:
                 mask_v = HMAC.new(nonce_k, mask_v, mhash).digest()
                 mask_t += mask_v
 
@@ -238,107 +276,67 @@ class DSS_SigScheme(object):
             nonce = self._bits2int(mask_t)
         return nonce
 
-    def sign(self, msg_hash):
-        """Produce the DSS signature of a message.
+    def _valid_hash(self, msg_hash):
+        return True
 
-        :Parameters:
-          msg_hash : hash object
-            The hash that was carried out over the message.
-            The object belongs to the `Crypto.Hash` package.
 
-            Under mode *'fips-186-3'*, the hash must be a FIPS
-            approved secure hash (SHA-1 or a member of the SHA-2 family).
+class FipsDsaSigScheme(DssSigScheme):
 
-        :Return: The signature encoded as a byte string.
-        :Raise ValueError:
-            If the hash algorithm is incompatible to the DSA key.
-        :Raise TypeError:
-            If the DSA key has no private half.
-        """
+    #: List of L (bit length of p) and N (bit length of q) combinations
+    #: that are allowed by FIPS 186-3. The security level is provided in
+    #: Table 2 of FIPS 800-57 (rev3).
+    _fips_186_3_L_N = (
+                        (1024, 160),    # 80 bits  (SHA-1 or stronger)
+                        (2048, 224),    # 112 bits (SHA-224 or stronger)
+                        (2048, 256),    # 128 bits (SHA-256 or stronger)
+                        (3072, 256)     # 256 bits (SHA-512)
+                      )
 
-        # Generate the nonce k (critical!)
-        if self._deterministic:
-            nonce = self._compute_nonce(msg_hash)
-        else:
-            if self._n > msg_hash.digest_size * 8:
-                raise ValueError("Hash is not long enough")
+    def __init__(self, key, encoding, order, randfunc):
+        super(FipsDsaSigScheme, self).__init__(key, encoding, order)
+        self._randfunc = randfunc
 
-            if not msg_hash.name.upper().startswith("SHA"):
-                raise ValueError("Hash %s does not belong to SHS" % msg_hash.name)
+        L = Integer(key.p).size_in_bits()
+        if (L, self._order_bits) not in self._fips_186_3_L_N:
+            error = ("L/N (%d, %d) is not compliant to FIPS 186-3"
+                     % (L, self._order_bits))
+            raise ValueError(error)
 
-            rng = StrongRandom(randfunc=self._randfunc)
-            nonce = rng.randint(1, self._key.q - 1)
+    def _compute_nonce(self, msg_hash):
+        # hash is not used
+        return Integer.random_range(min_inclusive=1,
+                                    max_exclusive=self._order,
+                                    randfunc=self._randfunc)
 
-        # Perform signature using the raw API
-        z = bytes_to_long(msg_hash.digest()[:self._n])
-        sig_pair = self._key._sign(z, nonce)
+    def _valid_hash(self, msg_hash):
+        """Verify that SHA-1, SHA-2 or SHA-3 are used"""
+        return (msg_hash.oid == "1.3.14.3.2.26" or
+                msg_hash.oid.startswith("2.16.840.1.101.3.4.2."))
 
-        # Encode the signature into a single byte string
-        if self._encoding == 'binary':
-            output = b("").join([long_to_bytes(x, self._n)
-                                 for x in sig_pair])
-        else:
-            # Dss-sig  ::=  SEQUENCE  {
-            #               r       OCTET STRING,
-            #               s       OCTET STRING
-            # }
-            der_seq = DerSequence(sig_pair)
-            output = der_seq.encode()
 
-        return output
+class FipsEcDsaSigScheme(DssSigScheme):
 
-    def verify(self, msg_hash, signature):
-        """Verify that a certain DSS signature is authentic.
+    def __init__(self, key, encoding, order, randfunc):
+        super(FipsEcDsaSigScheme, self).__init__(key, encoding, order)
+        self._randfunc = randfunc
 
-        This function checks if the party holding the private half of the key
-        really signed the message.
+    def _compute_nonce(self, msg_hash):
+        return Integer.random_range(min_inclusive=1,
+                                    max_exclusive=_curve.order,
+                                    randfunc=self._randfunc)
 
-        :Parameters:
-          msg_hash : hash object
-            The hash that was carried out over the message.
-            This is an object belonging to the `Crypto.Hash` module.
+    def _valid_hash(self, msg_hash):
+        """Verify that SHA-[23] (256|384|512) bits are used to
+        match the 128-bit security of P-256"""
 
-            Under mode *'fips-186-3'*, the hash must be a FIPS
-            approved secure hash (SHA-1 or a member of the SHA-2 family).
+        approved = ("2.16.840.1.101.3.4.2.1",
+                    "2.16.840.1.101.3.4.2.2",
+                    "2.16.840.1.101.3.4.2.3",
+                    "2.16.840.1.101.3.4.2.8",
+                    "2.16.840.1.101.3.4.2.9",
+                    "2.16.840.1.101.3.4.2.10")
 
-          signature : byte string
-            The signature that needs to be validated.
-
-        :Raise ValueError:
-            If the signature is not authentic.
-        """
-
-        if not self._deterministic:
-            if self._n > msg_hash.digest_size * 8:
-                raise ValueError("Hash is not long enough")
-            if not msg_hash.name.lower().startswith("sha"):
-                raise ValueError("Hash %s does not belong to SHS" % msg_hash.name)
-
-        if self._encoding == 'binary':
-            if len(signature) != (2 * self._n):
-                raise ValueError("The signature is not authentic")
-            r_prime, s_prime = [bytes_to_long(x)
-                                for x in (signature[:self._n],
-                                          signature[self._n:])]
-        else:
-            try:
-                der_seq = DerSequence()
-                der_seq.decode(signature)
-            except (ValueError, IndexError):
-                raise ValueError("The signature is not authentic")
-            if len(der_seq) != 2 or not der_seq.hasOnlyInts():
-                raise ValueError("The signature is not authentic")
-            r_prime, s_prime = der_seq[0], der_seq[1]
-
-        if not (0 < r_prime < self._key.q) or not (0 < s_prime < self._key.q):
-            raise ValueError("The signature is not authentic")
-
-        z = bytes_to_long(msg_hash.digest()[:self._n])
-        result = self._key._verify(z, (r_prime, s_prime))
-        if not result:
-            raise ValueError("The signature is not authentic")
-        # Make PyCrypto code to fail
-        return False
+        return msg_hash.oid in approved
 
 
 def new(key, mode, encoding='binary', randfunc=None):
@@ -346,15 +344,15 @@ def new(key, mode, encoding='binary', randfunc=None):
     can be used to perform DSS signature or verification.
 
     :Parameters:
-      key : a DSA key object
+      key : a `Crypto.PublicKey.DSA` or `Crypto.PublicKey.ECC` key object
         If the key has got its private half, both signature and
         verification are possible.
 
         If it only has the public half, verification is possible
         but not signature generation.
 
-        If *L* and *N* are the bit lengths of the modules *p* and *q*,
-        the combination *(L,N)* must appear in the following list,
+        For DSA keys, let *L* and *N* be the bit lengths of the modules *p*
+        and *q*: the combination *(L,N)* must appear in the following list,
         in compliance to section 4.2 of `FIPS-186`__:
 
         - (1024, 160)
@@ -377,7 +375,7 @@ def new(key, mode, encoding='binary', randfunc=None):
 
         The following values are accepted:
 
-        - *'binary'*, the signature is the raw concatenation
+        - *'binary'* (default), the signature is the raw concatenation
           of *r* and *s*. The size in bytes of the signature is always
           two times the size of *q*.
 
@@ -386,11 +384,40 @@ def new(key, mode, encoding='binary', randfunc=None):
 
       randfunc : callable
         The source of randomness. If ``None``, the internal RNG is used.
-        It is not used under mode *'deterministic-rfc6979'*.
+        Only used for the *'fips-186-3'* mode.
 
     .. __: http://csrc.nist.gov/publications/fips/fips186-3/fips_186-3.pdf
     .. __: http://csrc.nist.gov/publications/fips/fips186-3/fips_186-3.pdf
     .. _RFC6979: http://tools.ietf.org/html/rfc6979
     """
 
-    return DSS_SigScheme(key, mode, encoding, randfunc)
+    # The goal of the 'mode' parameter is to avoid to
+    # have the current version of the standard as default.
+    #
+    # Over time, such version will be superseded by (for instance)
+    # FIPS 186-4 and it will be odd to have -3 as default.
+
+    if encoding not in ('binary', 'der'):
+        raise ValueError("Unknown encoding '%s'" % encoding)
+
+    if isinstance(key, EccKey):
+        order = _curve.order
+        private_key_attr = 'd'
+    else:
+        order = Integer(key.q)
+        private_key_attr = 'x'
+
+    if key.has_private():
+        private_key = getattr(key, private_key_attr)
+    else:
+        private_key = None
+
+    if mode == 'deterministic-rfc6979':
+        return DeterministicDsaSigScheme(key, encoding, order, private_key)
+    elif mode == 'fips-186-3':
+        if isinstance(key, EccKey):
+            return FipsEcDsaSigScheme(key, encoding, order, randfunc)
+        else:
+            return FipsDsaSigScheme(key, encoding, order, randfunc)
+    else:
+        raise ValueError("Unknown DSS mode '%s'" % mode)
