@@ -28,48 +28,54 @@ objects.
 
 """
 
-from Crypto.Util.py3compat import *
+from Crypto.Util.py3compat import byte_string, b, bchr, bord
 
 from Crypto.Util.number import long_to_bytes, bytes_to_long
 
-__all__ = [ 'DerObject', 'DerInteger', 'DerOctetString', 'DerNull',
-            'DerSequence', 'DerObjectId', 'DerBitString', 'DerSetOf',
-            'newDerInteger', 'newDerOctetString', 'newDerSequence',
-            'newDerObjectId', 'newDerBitString', 'newDerSetOf' ]
+__all__ = ['DerObject', 'DerInteger', 'DerOctetString', 'DerNull',
+           'DerSequence', 'DerObjectId', 'DerBitString', 'DerSetOf']
 
-def _isInt(x, onlyNonNegative=False):
+
+def _is_number(x, only_non_negative=False):
     test = 0
     try:
-        test += x
+        test = x + test
     except TypeError:
         return False
-    return not onlyNonNegative or x>=0
+    return not only_non_negative or x >= 0
 
-class BytesIO_EOF(BytesIO):
-    """This class differs from BytesIO in that an EOFError exception is
+
+class BytesIO_EOF(object):
+    """This class differs from BytesIO in that a ValueError exception is
     raised whenever EOF is reached."""
 
-    def __init__(self, *params):
-        BytesIO.__init__(self, *params)
-        self.setRecord(False)
+    def __init__(self, initial_bytes):
+        self._buffer = initial_bytes
+        self._index = 0
+        self._bookmark  = None
 
-    def setRecord(self, record):
-        self._record = record
-        self._recording = b("")
+    def set_bookmark(self):
+        self._bookmark = self._index
+
+    def data_since_bookmark(self):
+        assert self._bookmark is not None
+        return self._buffer[self._bookmark:self._index]
+
+    def remaining_data(self):
+        return len(self._buffer) - self._index
 
     def read(self, length):
-        s = BytesIO.read(self, length)
-        if len(s)<length:
-            raise EOFError
-        if self._record:
-            self._recording += s
-        return s
+        new_index = self._index + length
+        if new_index > len(self._buffer):
+            raise ValueError
+
+        result = self._buffer[self._index:new_index]
+        self._index = new_index
+        return result
 
     def read_byte(self):
-        return self.read(1)[0]
+        return bord(self.read(1)[0])
 
-class _NoDerElementError(EOFError):
-    pass
 
 class DerObject(object):
         """Base class for defining a single DER object.
@@ -77,93 +83,119 @@ class DerObject(object):
         This class should never be directly instantiated.
         """
 
-        def __init__(self, asn1Id=None, payload=b(''), implicit=None, constructed=False):
+        def __init__(self, asn1Id=None, payload=b(''), implicit=None,
+                     constructed=False, explicit=None):
                 """Initialize the DER object according to a specific ASN.1 type.
 
                 :Parameters:
                   asn1Id : integer
-                    The universal DER tag identifier for this object
-                    (e.g. 0x10 for a SEQUENCE). If None, the tag is not known
-                    yet.
+                    The universal DER tag number for this object
+                    (e.g. 0x10 for a SEQUENCE).
+                    If None, the tag is not known yet.
 
                   payload : byte string
-                    The initial payload of the object.
+                    The initial payload of the object (that it,
+                    the content octets).
                     If not specified, the payload is empty.
 
                   implicit : integer
-                    The IMPLICIT tag to use for the encoded object.
+                    The IMPLICIT tag number to use for the encoded object.
                     It overrides the universal tag *asn1Id*.
 
                   constructed : bool
                     True when the ASN.1 type is *constructed*.
                     False when it is *primitive*.
+
+                  explicit : integer
+                    The EXPLICIT tag number to use for the encoded object.
                 """
 
-                if asn1Id==None:
-                    self._idOctet = None
+                if asn1Id is None:
+                    # The tag octet will be read in with ``decode``
+                    self._tag_octet = None
                     return
                 asn1Id = self._convertTag(asn1Id)
-                self._implicit = implicit
-                if implicit:
-                    # In a BER/DER identifier octet:
-                    # * bits 4-0 contain the tag value
-                    # * bit 5 is set if the type is 'construted'
-                    #   and unset if 'primitive'
-                    # * bits 7-6 depend on the encoding class
-                    #
-                    # Class        | Bit 7, Bit 6
-                    # universal    |   0      0
-                    # application  |   0      1
-                    # context-spec |   1      0 (default for IMPLICIT)
-                    # private      |   1      1
-                    #
-                    self._idOctet = 0x80 | self._convertTag(implicit)
+
+                # In a BER/DER identifier octet:
+                # * bits 4-0 contain the tag value
+                # * bit 5 is set if the type is 'constructed'
+                #   and unset if 'primitive'
+                # * bits 7-6 depend on the encoding class
+                #
+                # Class        | Bit 7, Bit 6
+                # ----------------------------------
+                # universal    |   0      0
+                # application  |   0      1
+                # context-spec |   1      0 (default for IMPLICIT/EXPLICIT)
+                # private      |   1      1
+                #
+                if explicit is None:
+                    if implicit is None:
+                        self._tag_octet = asn1Id
+                    else:
+                        self._tag_octet = 0x80 | self._convertTag(implicit)
+                    self._tag_octet |= 0x20 * constructed
                 else:
-                    self._idOctet = asn1Id
-                if constructed:
-                    self._idOctet |= 0x20
+                    if implicit is None:
+                        self._tag_octet = 0xA0 | self._convertTag(explicit)
+                    else:
+                        raise ValueError("Explicit and implicit tags are"
+                                         " mutually exclusive")
+                    self._inner_tag_octet = asn1Id + 0x20 * constructed
+
                 self.payload = payload
 
         def _convertTag(self, tag):
                 """Check if *tag* is a real DER tag.
                 Convert it from a character to number if necessary.
                 """
-                if not _isInt(tag):
-                    if len(tag)==1:
+                if not _is_number(tag):
+                    if len(tag) == 1:
                         tag = bord(tag[0])
                 # Ensure that tag is a low tag
-                if not (_isInt(tag) and 0 <= tag < 0x1F):
+                if not (_is_number(tag) and 0 <= tag < 0x1F):
                     raise ValueError("Wrong DER tag")
                 return tag
 
-        def _lengthOctets(self):
-                """Build length octets according to the current object's payload.
-
-                Return a byte string that encodes the payload length (in
-                bytes) in a format suitable for DER length octets (L).
+        @staticmethod
+        def _definite_form(length):
+                """Build length octets according to BER/DER
+                definite form.
                 """
-                payloadLen = len(self.payload)
-                if payloadLen>127:
-                        encoding = long_to_bytes(payloadLen)
-                        return bchr(len(encoding)+128) + encoding
-                return bchr(payloadLen)
+                if length > 127:
+                        encoding = long_to_bytes(length)
+                        return bchr(len(encoding) + 128) + encoding
+                return bchr(length)
 
         def encode(self):
                 """Return this DER element, fully encoded as a binary byte string."""
+
                 # Concatenate identifier octets, length octets,
                 # and contents octets
-                return bchr(self._idOctet) + self._lengthOctets() + self.payload
+
+                output_payload = self.payload
+
+                # In case of an EXTERNAL tag, first encode the inner
+                # element.
+                if hasattr(self, "_inner_tag_octet"):
+                    output_payload = (bchr(self._inner_tag_octet) +
+                                      self._definite_form(len(self.payload)) +
+                                      self.payload)
+
+                return (bchr(self._tag_octet) +
+                        self._definite_form(len(output_payload)) +
+                        output_payload)
 
         def _decodeLen(self, s):
                 """Decode DER length octets from a file."""
 
-                length = bord(s.read_byte())
-                if length<=127:
+                length = s.read_byte()
+                if length <= 127:
                         return length
                 payloadLength = bytes_to_long(s.read(length & 0x7F))
                 # According to DER (but not BER) the long form is used
                 # only when the length doesn't fit into 7 bits.
-                if payloadLength<=127:
+                if payloadLength <= 127:
                         raise ValueError("Not a DER length tag (but still valid BER).")
                 return payloadLength
 
@@ -177,34 +209,47 @@ class DerObject(object):
 
                 :Raise ValueError:
                   In case of parsing errors.
-                :Raise EOFError:
-                  If the DER element is too short.
                 """
 
+                if not byte_string(derEle):
+                    raise ValueError("Input is not a byte string")
+
                 s = BytesIO_EOF(derEle)
                 self._decodeFromStream(s)
+
                 # There shouldn't be other bytes left
-                try:
-                    b = s.read_byte()
+                if s.remaining_data() > 0:
                     raise ValueError("Unexpected extra data after the DER structure")
-                except EOFError:
-                    pass
+
+                return self
 
         def _decodeFromStream(self, s):
                 """Decode a complete DER element from a file."""
 
-                try:
-                    idOctet = bord(s.read_byte())
-                except EOFError:
-                    raise _NoDerElementError
-                if self._idOctet != None:
-                    if idOctet != self._idOctet:
+                idOctet = s.read_byte()
+                if self._tag_octet is not None:
+                    if idOctet != self._tag_octet:
                         raise ValueError("Unexpected DER tag")
                 else:
-                    self._idOctet = idOctet
+                    self._tag_octet = idOctet
                 length = self._decodeLen(s)
                 self.payload = s.read(length)
 
+                # In case of an EXTERNAL tag, further decode the inner
+                # element.
+                if hasattr(self, "_inner_tag_octet"):
+                    p = BytesIO_EOF(self.payload)
+                    inner_octet = p.read_byte()
+                    if inner_octet != self._inner_tag_octet:
+                        raise ValueError("Unexpected internal DER tag")
+                    length = self._decodeLen(p)
+                    self.payload = p.read(length)
+
+                    # There shouldn't be other bytes left
+                    if p.remaining_data() > 0:
+                        raise ValueError("Unexpected extra data after the DER structure")
+
+
 class DerInteger(DerObject):
         """Class to model a DER INTEGER.
 
@@ -224,13 +269,13 @@ class DerInteger(DerObject):
           >>>   int_der = DerInteger()
           >>>   int_der.decode(s)
           >>>   print int_der.value
-          >>> except (ValueError, EOFError):
+          >>> except ValueError:
           >>>   print "Not a valid DER INTEGER"
 
         the output will be ``9``.
         """
 
-        def __init__(self, value=0, implicit=None):
+        def __init__(self, value=0, implicit=None, explicit=None):
                 """Initialize the DER object as an INTEGER.
 
                 :Parameters:
@@ -242,8 +287,9 @@ class DerInteger(DerObject):
                     It overrides the universal tag for INTEGER (2).
                 """
 
-                DerObject.__init__(self, 0x02, b(''), implicit, False)
-                self.value = value #: The integer value
+                DerObject.__init__(self, 0x02, b(''), implicit,
+                                   False, explicit)
+                self.value = value  #: The integer value
 
         def encode(self):
                 """Return the DER INTEGER, fully encoded as a
@@ -252,7 +298,7 @@ class DerInteger(DerObject):
                 number = self.value
                 self.payload = b('')
                 while True:
-                    self.payload = bchr(number&255) + self.payload
+                    self.payload = bchr(int(number & 255)) + self.payload
                     if 128 <= number <= 255:
                         self.payload = bchr(0x00) + self.payload
                     if -128 <= number <= 255:
@@ -270,10 +316,9 @@ class DerInteger(DerObject):
 
                 :Raise ValueError:
                   In case of parsing errors.
-                :Raise EOFError:
-                  If the DER element is too short.
                 """
-                DerObject.decode(self, derEle)
+
+                return DerObject.decode(self, derEle)
 
         def _decodeFromStream(self, s):
                 """Decode a complete DER INTEGER from a file."""
@@ -291,11 +336,6 @@ class DerInteger(DerObject):
                 if self.payload and bord(self.payload[0]) & 0x80:
                     self.value -= bits
 
-def newDerInteger(number):
-    """Create a DerInteger object, already initialized with an integer."""
-
-    der = DerInteger(number)
-    return der
 
 class DerSequence(DerObject):
         """Class to model a DER SEQUENCE.
@@ -329,7 +369,7 @@ class DerSequence(DerObject):
           >>>   print len(seq_der)
           >>>   print seq_der[0]
           >>>   print seq_der[:]
-          >>> except (ValueError, EOFError):
+          >>> except ValueError:
           >>>   print "Not a valid DER SEQUENCE"
 
         the output will be::
@@ -354,47 +394,55 @@ class DerSequence(DerObject):
                 """
 
                 DerObject.__init__(self, 0x10, b(''), implicit, True)
-                if startSeq==None:
+                if startSeq is None:
                     self._seq = []
                 else:
                     self._seq = startSeq
 
-        ## A few methods to make it behave like a python sequence
+        # A few methods to make it behave like a python sequence
 
         def __delitem__(self, n):
                 del self._seq[n]
+
         def __getitem__(self, n):
                 return self._seq[n]
+
         def __setitem__(self, key, value):
                 self._seq[key] = value
-        def __setslice__(self,i,j,sequence):
+
+        def __setslice__(self, i, j, sequence):
                 self._seq[i:j] = sequence
-        def __delslice__(self,i,j):
+
+        def __delslice__(self, i, j):
                 del self._seq[i:j]
+
         def __getslice__(self, i, j):
                 return self._seq[max(0, i):max(0, j)]
+
         def __len__(self):
                 return len(self._seq)
+
         def __iadd__(self, item):
                 self._seq.append(item)
                 return self
+
         def append(self, item):
                 self._seq.append(item)
                 return self
 
-        def hasInts(self, onlyNonNegative=True):
+        def hasInts(self, only_non_negative=True):
                 """Return the number of items in this sequence that are
                 integers.
 
                 :Parameters:
-                  onlyNonNegative : boolean
+                  only_non_negative : boolean
                     If True, negative integers are not counted in.
                 """
-                def _isInt2(x):
-                    return _isInt(x, onlyNonNegative)
-                return len(filter(_isInt2, self._seq))
+                def _is_number2(x):
+                    return _is_number(x, only_non_negative)
+                return len(filter(_is_number2, self._seq))
 
-        def hasOnlyInts(self, onlyNonNegative=True):
+        def hasOnlyInts(self, only_non_negative=True):
                 """Return True if all items in this sequence are integers
                 or non-negative integers.
 
@@ -402,10 +450,10 @@ class DerSequence(DerObject):
                 or at least one member is not an integer.
 
                 :Parameters:
-                  onlyNonNegative : boolean
+                  only_non_negative : boolean
                     If True, the presence of negative integers
                     causes the method to return False."""
-                return self._seq and self.hasInts(onlyNonNegative)==len(self._seq)
+                return self._seq and self.hasInts(only_non_negative) == len(self._seq)
 
         def encode(self):
                 """Return this DER SEQUENCE, fully encoded as a
@@ -417,32 +465,40 @@ class DerSequence(DerObject):
                 """
                 self.payload = b('')
                 for item in self._seq:
-                    try:
+                    if byte_string(item):
                         self.payload += item
-                    except TypeError:
-                        try:
-                            self.payload += DerInteger(item).encode()
-                        except TypeError:
-                            raise ValueError("Trying to DER encode an unknown object")
+                    elif _is_number(item):
+                        self.payload += DerInteger(item).encode()
+                    else:
+                        self.payload += item.encode()
                 return DerObject.encode(self)
 
-        def decode(self, derEle):
+        def decode(self, derEle, nr_elements=None, only_ints_expected=False):
                 """Decode a complete DER SEQUENCE, and re-initializes this
                 object with it.
 
                 :Parameters:
                   derEle : byte string
                     A complete SEQUENCE DER element.
+                  nr_elements : None, integer or list of integers
+                    The number of members the SEQUENCE can have
+                  only_ints_expected : boolean
+                    Whether the SEQUENCE is expected to contain only integers.
 
                 :Raise ValueError:
                   In case of parsing errors.
-                :Raise EOFError:
-                  If the DER element is too short.
 
                 DER INTEGERs are decoded into Python integers. Any other DER
                 element is not decoded. Its validity is not checked.
                 """
-                DerObject.decode(self, derEle)
+
+                self._nr_elements = nr_elements
+                result = DerObject.decode(self, derEle)
+
+                if only_ints_expected and not self.hasOnlyInts():
+                    raise ValueError("Some members are not INTEGERs")
+
+                return result
 
         def _decodeFromStream(self, s):
                 """Decode a complete DER SEQUENCE from a file."""
@@ -454,35 +510,31 @@ class DerSequence(DerObject):
 
                 # Add one item at a time to self.seq, by scanning self.payload
                 p = BytesIO_EOF(self.payload)
-                while True:
+                while p.remaining_data() > 0:
+                    p.set_bookmark()
+
+                    der = DerObject()
+                    der._decodeFromStream(p)
+
+                    # Parse INTEGERs differently
+                    if der._tag_octet != 0x02:
+                        self._seq.append(p.data_since_bookmark())
+                    else:
+                        derInt = DerInteger()
+                        derInt.decode(p.data_since_bookmark())
+                        self._seq.append(derInt.value)
+
+                ok = True
+                if self._nr_elements is not None:
                     try:
-                        p.setRecord(True)
-                        der = DerObject()
-                        der._decodeFromStream(p)
+                        ok = len(self._seq) in self._nr_elements
+                    except TypeError:
+                        ok = len(self._seq) == self._nr_elements
 
-                        # Parse INTEGERs differently
-                        if der._idOctet != 0x02:
-                            self._seq.append(p._recording)
-                        else:
-                            derInt = DerInteger()
-                            derInt.decode(p._recording)
-                            self._seq.append(derInt.value)
+                if not ok:
+                    raise ValueError("Unexpected number of members (%d)"
+                                     " in the sequence" % len(self._seq))
 
-                    except _NoDerElementError:
-                        break
-                # end
-
-def newDerSequence(*der_objs):
-    """Create a DerSequence object, already initialized with all objects
-    passed as parameters."""
-
-    der = DerSequence()
-    for obj in der_objs:
-        if isinstance(obj, DerObject):
-            der += obj.encode()
-        else:
-            der += obj
-    return der
 
 class DerOctetString(DerObject):
     """Class to model a DER OCTET STRING.
@@ -505,7 +557,7 @@ class DerOctetString(DerObject):
     >>>   os_der = DerOctetString()
     >>>   os_der.decode(s)
     >>>   print hexlify(os_der.payload)
-    >>> except (ValueError, EOFError):
+    >>> except ValueError:
     >>>   print "Not a valid DER OCTET STRING"
 
     the output will be ``aabb``.
@@ -525,15 +577,6 @@ class DerOctetString(DerObject):
         """
         DerObject.__init__(self, 0x04, value, implicit, False)
 
-def newDerOctetString(binstring):
-    """Create a DerOctetString object, already initialized with the binary
-    string."""
-
-    if isinstance(binstring, DerObject):
-        der = DerOctetString(binstring.encode())
-    else:
-        der = DerOctetString(binstring)
-    return der
 
 class DerNull(DerObject):
     """Class to model a DER NULL element."""
@@ -541,7 +584,8 @@ class DerNull(DerObject):
     def __init__(self):
         """Initialize the DER object as a NULL."""
 
-        DerObject.__init__(self, 0x05, b(''), False)
+        DerObject.__init__(self, 0x05, b(''), None, False)
+
 
 class DerObjectId(DerObject):
     """Class to model a DER OBJECT ID.
@@ -564,13 +608,13 @@ class DerObjectId(DerObject):
     >>>   oid_der = DerObjectId()
     >>>   oid_der.decode(s)
     >>>   print oid_der.value
-    >>> except (ValueError, EOFError):
+    >>> except ValueError:
     >>>   print "Not a valid DER OBJECT ID"
 
     the output will be ``1.2.840.113549.1.1.1``.
     """
 
-    def __init__(self, value='', implicit=None):
+    def __init__(self, value='', implicit=None, explicit=None):
         """Initialize the DER object as an OBJECT ID.
 
         :Parameters:
@@ -579,16 +623,18 @@ class DerObjectId(DerObject):
           implicit : integer
             The IMPLICIT tag to use for the encoded object.
             It overrides the universal tag for OBJECT ID (6).
+          explicit : integer
+            The EXPLICIT tag to use for the encoded object.
         """
-        DerObject.__init__(self, 0x06, b(''), implicit, False)
-        self.value = value #: The Object ID, a dot separated list of integers
+        DerObject.__init__(self, 0x06, b(''), implicit, False, explicit)
+        self.value = value  #: The Object ID, a dot separated list of integers
 
     def encode(self):
         """Return the DER OBJECT ID, fully encoded as a
         binary string."""
 
-        comps = map(int,self.value.split("."))
-        if len(comps)<2:
+        comps = map(int, self.value.split("."))
+        if len(comps) < 2:
             raise ValueError("Not a valid Object Identifier string")
         self.payload = bchr(40*comps[0]+comps[1])
         for v in comps[2:]:
@@ -610,11 +656,9 @@ class DerObjectId(DerObject):
 
         :Raise ValueError:
             In case of parsing errors.
-        :Raise EOFError:
-            If the DER element is too short.
         """
 
-        DerObject.decode(self, derEle)
+        return DerObject.decode(self, derEle)
 
     def _decodeFromStream(self, s):
         """Decode a complete DER OBJECT ID from a file."""
@@ -624,25 +668,16 @@ class DerObjectId(DerObject):
 
         # Derive self.value from self.payload
         p = BytesIO_EOF(self.payload)
-        comps = list(map(str, divmod(bord(p.read_byte()),40)))
+        comps = list(map(str, divmod(p.read_byte(), 40)))
         v = 0
-        try:
-            while True:
-                c = p.read_byte()
-                v = v*128 + (bord(c) & 0x7F)
-                if not (bord(c) & 0x80):
-                    comps.append(str(v))
-                    v = 0
-        except EOFError:
-            pass
+        while p.remaining_data():
+            c = p.read_byte()
+            v = v*128 + (c & 0x7F)
+            if not (c & 0x80):
+                comps.append(str(v))
+                v = 0
         self.value = '.'.join(comps)
 
-def newDerObjectId(dottedstring):
-    """Create a DerObjectId object, already initialized with the given Object
-    Identifier (a dotted string)."""
-
-    der = DerObjectId(dottedstring)
-    return der
 
 class DerBitString(DerObject):
     """Class to model a DER BIT STRING.
@@ -665,25 +700,32 @@ class DerBitString(DerObject):
     >>>   bs_der = DerBitString()
     >>>   bs_der.decode(s)
     >>>   print hexlify(bs_der.value)
-    >>> except (ValueError, EOFError):
+    >>> except ValueError:
     >>>   print "Not a valid DER OCTET STRING"
 
     the output will be ``aabb``.
     """
 
-    def __init__(self, value=b(''), implicit=None):
+    def __init__(self, value=b(''), implicit=None, explicit=None):
         """Initialize the DER object as a BIT STRING.
 
         :Parameters:
-          value : byte string
+          value : byte string or DER object
             The initial, packed bit string.
             If not specified, the bit string is empty.
           implicit : integer
             The IMPLICIT tag to use for the encoded object.
             It overrides the universal tag for OCTET STRING (3).
+          explicit : integer
+            The EXPLICIT tag to use for the encoded object.
         """
-        DerObject.__init__(self, 0x03, b(''), implicit, False)
-        self.value = value #: The bitstring value (packed)
+        DerObject.__init__(self, 0x03, b(''), implicit, False, explicit)
+
+        # The bitstring value (packed)
+        if isinstance(value, DerObject):
+            self.value = value.encode()
+        else:
+            self.value = value
 
     def encode(self):
         """Return the DER BIT STRING, fully encoded as a
@@ -703,11 +745,9 @@ class DerBitString(DerObject):
 
         :Raise ValueError:
             In case of parsing errors.
-        :Raise EOFError:
-            If the DER element is too short.
         """
 
-        DerObject.decode(self, derEle)
+        return DerObject.decode(self, derEle)
 
     def _decodeFromStream(self, s):
         """Decode a complete DER BIT STRING DER from a file."""
@@ -715,7 +755,7 @@ class DerBitString(DerObject):
         # Fill-up self.payload
         DerObject._decodeFromStream(self, s)
 
-        if self.payload and bord(self.payload[0])!=0:
+        if self.payload and bord(self.payload[0]) != 0:
             raise ValueError("Not a valid BIT STRING")
 
         # Fill-up self.value
@@ -724,15 +764,6 @@ class DerBitString(DerObject):
         if self.payload:
             self.value = self.payload[1:]
 
-def newDerBitString(binstring):
-    """Create a DerStringString object, already initialized with the binary
-    string."""
-
-    if isinstance(binstring, DerObject):
-        der = DerBitString(binstring.encode())
-    else:
-        der = DerBitString(binstring)
-    return der
 
 class DerSetOf(DerObject):
     """Class to model a DER SET OF.
@@ -755,7 +786,7 @@ class DerSetOf(DerObject):
     >>>   so_der = DerSetOf()
     >>>   so_der.decode(s)
     >>>   print [x for x in so_der]
-    >>> except (ValueError, EOFError):
+    >>> except ValueError:
     >>>   print "Not a valid DER SET OF"
 
     the output will be ``[4, 5, 6]``.
@@ -773,7 +804,11 @@ class DerSetOf(DerObject):
         """
         DerObject.__init__(self, 0x11, b(''), implicit, True)
         self._seq = []
+
+        # All elements must be of the same type (and therefore have the
+        # same leading octet)
         self._elemOctet = None
+
         if startSet:
             for e in startSet:
                 self.add(e)
@@ -795,15 +830,20 @@ class DerSetOf(DerObject):
               An element of the same type of objects already in the set.
               It can be an integer or a DER encoded object.
         """
-        if _isInt(elem):
+
+        if _is_number(elem):
             eo = 0x02
+        elif isinstance(elem, DerObject):
+            eo = self._tag_octet
         else:
             eo = bord(elem[0])
+
         if self._elemOctet != eo:
-            if self._elemOctet:
+            if self._elemOctet is not None:
                 raise ValueError("New element does not belong to the set")
             self._elemOctet = eo
-        if not elem in self._seq:
+
+        if elem not in self._seq:
             self._seq.append(elem)
 
     def decode(self, derEle):
@@ -819,11 +859,9 @@ class DerSetOf(DerObject):
 
         :Raise ValueError:
             In case of parsing errors.
-        :Raise EOFError:
-            If the DER element is too short.
         """
 
-        DerObject.decode(self, derEle)
+        return DerObject.decode(self, derEle)
 
     def _decodeFromStream(self, s):
         """Decode a complete DER SET OF from a file."""
@@ -836,29 +874,26 @@ class DerSetOf(DerObject):
         # Add one item at a time to self.seq, by scanning self.payload
         p = BytesIO_EOF(self.payload)
         setIdOctet = -1
-        while True:
-            try:
-                p.setRecord(True)
-                der = DerObject()
-                der._decodeFromStream(p)
+        while p.remaining_data() > 0:
+            p.set_bookmark()
 
-                # Verify that all members are of the same type
-                if setIdOctet < 0:
-                    setIdOctet = der._idOctet
-                else:
-                    if setIdOctet != der._idOctet:
-                        raise ValueError("Not all elements are of the same DER type")
+            der = DerObject()
+            der._decodeFromStream(p)
 
-                # Parse INTEGERs differently
-                if setIdOctet != 0x02:
-                    self._seq.append(p._recording)
-                else:
-                    derInt = DerInteger()
-                    derInt.decode(p._recording)
-                    self._seq.append(derInt.value)
+            # Verify that all members are of the same type
+            if setIdOctet < 0:
+                setIdOctet = der._tag_octet
+            else:
+                if setIdOctet != der._tag_octet:
+                    raise ValueError("Not all elements are of the same DER type")
 
-            except _NoDerElementError:
-                break
+            # Parse INTEGERs differently
+            if setIdOctet != 0x02:
+                self._seq.append(p.data_since_bookmark())
+            else:
+                derInt = DerInteger()
+                derInt.decode(p.data_since_bookmark())
+                self._seq.append(derInt.value)
         # end
 
     def encode(self):
@@ -869,23 +904,13 @@ class DerSetOf(DerObject):
         # Elements in the set must be ordered in lexicographic order
         ordered = []
         for item in self._seq:
-            if _isInt(item):
+            if _is_number(item):
                 bys = DerInteger(item).encode()
+            elif isinstance(item, DerObject):
+                bys = item.encode()
             else:
                 bys = item
             ordered.append(bys)
         ordered.sort()
         self.payload = b('').join(ordered)
         return DerObject.encode(self)
-
-def newDerSetOf(*der_objs):
-    """Create a DerSequence object, already initialized with all objects
-    passed as parameters."""
-
-    der = DerSetOf()
-    for obj in der_objs:
-        if isinstance(obj, DerObject):
-            der.add(obj.encode())
-        else:
-            der.add(obj)
-    return der
