@@ -24,7 +24,6 @@
 # ===================================================================
 #
 
-from Crypto.pct_warnings import GetRandomNumber_DeprecationWarning, PowmInsecureWarning
 from warnings import warn as _warn
 import math
 import sys
@@ -51,7 +50,7 @@ except ImportError:
 
 # You need libgmp v5 or later to get mpz_powm_sec.  Warn if it's not available.
 if _fastmath is not None and not (hasattr(_fastmath, "HAVE_DECL_MPZ_POWM_SEC") and _fastmath.HAVE_DECL_MPZ_POWM_SEC):
-    _warn("Not using mpz_powm_sec.  You should rebuild using libgmp >= 5 to avoid timing attack vulnerability.", PowmInsecureWarning)
+    _warn("Not using mpz_powm_sec.  You should rebuild using libgmp >= 5 to avoid timing attack vulnerability.", UserWarning)
 
 # New functions
 from _number_new import *
@@ -78,12 +77,6 @@ def size (N):
         bits += 1
     return bits
 
-def getRandomNumber(N, randfunc=None):
-    """Deprecated.  Use getRandomInteger or getRandomNBitInteger instead."""
-    warnings.warn("Crypto.Util.number.getRandomNumber has confusing semantics"+
-    "and has been deprecated.  Use getRandomInteger or getRandomNBitInteger instead.",
-        GetRandomNumber_DeprecationWarning)
-    return getRandomNBitInteger(N, randfunc)
 
 def getRandomInteger(N, randfunc=None):
     """getRandomInteger(N:int, randfunc:callable):long
@@ -399,7 +392,7 @@ def long_to_bytes(n, blocksize=0):
     """
     # after much testing, this algorithm was deemed to be the fastest
     s = b('')
-    n = long(n)
+    n = int(n)
     pack = struct.pack
     while n > 0:
         s = pack('>I', n & 0xffffffffL) + s
